#include <ranges>
#include <concepts>
#include <string>

auto getLonger( std::ranges::range auto range, size_t length ) {
    return range | std::views::filter( [length]( const auto &v ) {
                      return std::ranges::size( v ) > length; } );
}

auto summarizePassing( std::ranges::range auto range, int limit ) {
    using namespace std::literals;
    return range | std::views::filter( [limit]( const auto &v ) {
                      return std::get< 1 >( v ) >= limit;
                  } )
                 | std::views::transform( []( const auto &v ) {
                      auto [ name, pts ] = v;
                      return name + ": "s + std::to_string( pts ) + " pts"s;
                  } );
}

auto everyNth( std::ranges::range auto range, size_t n ) {
    return range | std::views::filter( [i = 0, n]( const auto & ) mutable {
                      return i++ % n == 0;
                  } );
}

// should generate the resulting sequence of given type
template< std::integral I = size_t >
auto diagSeq() {
    return std::views::iota( I(0) )
        | std::views::transform( []( const I &v ) {
              return std::views::iota( I(0), v + 1 );
          } )
        | std::views::join;
}

// should generate the resulting sequence of given type
template< std::integral I = size_t >
auto diagSums() {
    return diagSeq< I >()
        | std::views::transform( [sum = I(0)]( const I &v ) mutable {
              sum += v;
              return sum;
           } );
}
